﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;

using Quartz;
using Quartz.Listener;

namespace TopshelfStudy.NetCoreDemo
{
    /// <summary>
    /// Quartz作业监听器
    /// </summary>
    public class CustomJobListener : IJobListener
    {
        public readonly NLog.Logger logger = NLog.LogManager.GetCurrentClassLogger();

        public virtual string Name => nameof(CustomJobListener);

        public CustomJobListener()
        {
        }

        /// <summary>
        /// 执行前
        /// </summary>
        public Task JobToBeExecuted(IJobExecutionContext context, CancellationToken cancellationToken = default(CancellationToken))
        {
            logger.Info("作业，执行前！");
            return Task.CompletedTask;
        }

        /// <summary>
        /// 执行后
        /// </summary>
        public Task JobWasExecuted(IJobExecutionContext context, JobExecutionException jobException, CancellationToken cancellationToken = default(CancellationToken))
        {
            logger.Info($"作业{context.JobDetail.Key.Name}，执行后!");
            return Task.CompletedTask;
        }

        /// <summary>
        /// 被否决执行
        /// </summary>
        public Task JobExecutionVetoed(IJobExecutionContext context, CancellationToken cancellationToken = default(CancellationToken))
        {
            logger.Info("作业，被否决执行！");

            return Task.CompletedTask;
        }
    }
}
